#   yadtreceiver
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    Provides the Receiver class which implements a twisted service
    application. The receiver allows to start processes triggered by
    broadcast events. It is configured using a dictionary. You can load
    configuration files using the load method from the configuration
    module. When activated in the configuration the receiver will send
    update notifications to a graphite server.
"""

__author__ = 'Arne Hilmann, Maximilien Riehl, Michael Gruber'
__version__ = '0.1.10'

import os
import traceback

from twisted.application import service
from twisted.internet import reactor
from twisted.python import log

from yadtbroadcastclient import WampBroadcaster

from yadtreceiver.graphite import send_update_notification_to_graphite
from yadtreceiver.protocols import ProcessProtocol
from yadtreceiver.events import Event



class ReceiverException(Exception):
    """
        To be raised when an exception occurs within the receiver.
    """


class Receiver(service.Service):
    """
        The receiver connects to the broadcaster and receives events
        for the targets that it subscribed to.
    """

    def get_target_directory(self, target):
        """
            Appends the given target name to the targets_directory.

            @raise ReceiverException: if the target directory does not exist.
        """
        hostname          = self.configuration['hostname']
        targets_directory = self.configuration['targets_directory']

        target_directory  = os.path.join(targets_directory, target)

        if not os.path.exists(target_directory):
            raise ReceiverException('(%s) target[%s] request failed: target directory "%s" does not exist.'
                                    % (hostname, target, target_directory))

        return target_directory


    def handle_request(self, event):
        """
            Handles a request for the given target by executing the given
            command (using the python_command and script_to_execute from
            the configuration).
        """
        self.publish_start(event)

        if self.configuration['graphite_active'] and len(event.arguments) > 0:
            action = event.arguments[0]
            self.notify_graphite(event.target, action)

        hostname          = str(self.configuration['hostname'])
        python_command    = str(self.configuration['python_command'])
        script_to_execute = str(self.configuration['script_to_execute'])

        command_and_arguments_list = [python_command, script_to_execute] + event.arguments
        command_with_arguments = ' '.join(command_and_arguments_list)

        process_protocol = ProcessProtocol(hostname, self.broadcaster, event.target, command_with_arguments)

        target_dir = self.get_target_directory(event.target)
        reactor.spawnProcess(process_protocol, python_command, command_and_arguments_list, env={}, path=target_dir)


    def notify_graphite(self, target, action):
        """
            Notifies the configured graphite server about events (update events).
        """
        if action == 'update':
            host = self.configuration['graphite_host']
            port = self.configuration['graphite_port']

            send_update_notification_to_graphite(target, host, port)


    def publish_failed(self, event, message):
        """
            Publishes a event to signal that the command on the target failed.
        """
        log.err(_stuff=Exception(message), _why=message)
        self.broadcaster.publish_cmd_for_target(event.target, event.command, events.FAILED, message)


    def publish_start(self, event):
        """
            Publishes a event to signal that the command on the target started.
        """
        hostname = self.configuration['hostname']
        message  = '(%s) target[%s] request: command="%s", arguments=%s' % (hostname, event.target, event.command, event.arguments)
        log.msg(message)
        self.broadcaster.publish_cmd_for_target(event.target, event.command, events.STARTED, message)


    def onConnect(self):
        """
            Subscribes to the targets from the configuration. The receiver
            is useless when no targets are configured, therefore it will exit
            with error code 1 when no targets are configured.
        """
        self.broadcaster.client.connectionLost = self.onConnectionLost

        host = self.configuration['broadcaster_host']
        port = self.configuration['broadcaster_port']

        log.msg('Successfully connected to broadcaster on %s:%s' % (host, port))

        targets = sorted(self.configuration['targets'])

        if not targets:
            log.err('No targets configured.')
            exit(1)

        for target in targets:
            log.msg('subscribing to target "%s".' % target)
            self.broadcaster.client.subscribe(target, self.onEvent)



    def _refresh_connection(self, delay=60 * 60, first_call=False):
        """
            When connected, closes connection to force a clean reconnect,
            except on first_call
        """
        reactor.callLater(delay, self._refresh_connection)
        if hasattr(self, 'broadcaster') and self.broadcaster.client:
            if not first_call:
                log.msg('Closing connection to broadcaster. This should force a connection-refresh.')
                self.broadcaster.client.sendClose()


    def onConnectionLost(self, reason):
        """
            Allows for clean reconnect behaviour, because it 'none'ifies 
            the client explicitly
        """
        log.err('connection lost: %s' % reason)
        self.broadcaster.client = None
        # TODO: superclass method should be called here?


    def _client_watchdog(self, delay=1):
        """
            Checks periodically if broadcaster.client is set
            (see also onConnectionLost), tries to reconnect after a 
            delay (delay after failed connect: 1, 2, 4, 8, 16, 32, 60, 60, ... 
            seconds)
        """
        if hasattr(self, 'broadcaster') and self.broadcaster.client:
            reactor.callLater(1, self._client_watchdog)
        else:
            reactor.callLater(delay, self._client_watchdog, min(60, 2 * delay))
            log.err('broadcast.client not set, trying to connect')
            if delay > 1:
                log.msg('(scheduling next try in %s seconds)' % delay)
            return self._connect_broadcaster()


    def onEvent(self, target, event_data):
        """
            Will be called when receiving an event from the broadcaster.
            See onConnect which subscribes to the targets.
        """
        event = Event(target, event_data)

        if event.is_a_request:
            try:
                self.handle_request(event)
            except Exception as e:
                log.err(e.message)

                for line in traceback.format_exc().splitlines():
                    log.err(line)

                self.publish_failed(event, e.message)

        else:
            log.msg(str(event))


    def set_configuration(self, configuration):
        """
            Assigns a configuration to this receiver instance.
        """
        self.configuration = configuration


    def startService(self):
        """
            Initializes logging and establishes connection to broadcaster.
        """
        log.msg('yadtreceiver version %s' % __version__)
        self._client_watchdog()
        self._refresh_connection(first_call=True)


    def stopService(self):
        """
            Writes 'shutting down service' to the log.
        """
        log.msg('shutting down service')


    def _connect_broadcaster(self):
        """
            Establishes a connection to the broadcaster as found in the
            configuration.
        """
        host = self.configuration['broadcaster_host']
        port = self.configuration['broadcaster_port']

        log.msg('Connecting to broadcaster on %s:%s' % (host, port))

        self.broadcaster = WampBroadcaster(host, port, 'yadtreceiver')
        self.broadcaster.addOnSessionOpenHandler(self.onConnect)
        self.broadcaster.connect()



#   yadtreceiver
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    Provides the ProcessProtocol.
"""

__author__ = 'Michael Gruber'

from twisted.internet import protocol
from twisted.python import log

from yadtreceiver import events


class ProcessProtocol(protocol.ProcessProtocol):
    def __init__(self, hostname, broadcaster, target, readable_command):
        """
            Initializes the process protocol with the given properties.
        """
        self.broadcaster = broadcaster
        self.hostname = hostname
        self.readable_command = readable_command
        self.target = target

        log.msg('(%s) target[%s] executing "%s"' % (self.hostname, target, readable_command))


    def processExited(self, reason):
        """
            publishes a finished-event when exit code of the execution is 0
            otherwise publishes a failed-event.
        """
        return_code = reason.value.exitCode

        if return_code != 0:
            self.publish_failed(return_code)
            return

        self.publish_finished()


    def publish_finished(self):
        """
            Uses the broadcaster-client to publish a finished-event.
        """
        message = '(%s) target[%s] request finished: "%s" succeeded.' \
                  % (self.hostname, self.target, self.readable_command)
        log.msg(message)
        self.broadcaster.publish_cmd_for_target(self.target, self.readable_command, events.FINISHED, message)


    def publish_failed(self, return_code):
        """
            Uses the broadcaster-client to publish a failed-event.
            The given return code will be included into the message of the event.
        """
        error_message = '(%s) target[%s] request "%s" failed: return code was %s.' \
                        % (self.hostname, self.target, self.readable_command, return_code)
        log.err(error_message)
        self.broadcaster.publish_cmd_for_target(self.target, self.readable_command, events.FAILED, error_message)



#   yadtreceiver
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    Use the function Configuration.load to read a Configuration from a file.

    example configuration file receiver.cfg:

        [receiver]
        log_filename = /var/log/yadtreceiver.log
        targets = devyadt, proyadt
        targets_directory = /etc/yadtshell/targets
        script_to_execute = /usr/bin/yadtshell

        [broadcaster]
        host = broadcaster.domain.tld
        port = 8081

        [graphite]
        active = yes
        host = graphite.domain.tld
        port = 2003
"""

__author__ = 'Michael Gruber'

import os
import socket
import sys

from ConfigParser import SafeConfigParser


DEFAULT_BROADCASTER_HOST = 'localhost'
DEFAULT_BROADCASTER_PORT = 8081

DEFAULT_GRAPHITE_HOST   = 'localhost'
DEFAULT_GRAPHITE_PORT   = 2003
DEFAULT_GRAPHITE_ACTIVE = False

DEFAULT_LOG_FILENAME      = '/var/log/yadtreceiver.log'
DEFAULT_PYTHON_COMMAND    = '/usr/bin/python'
DEFAULT_SCRIPT_TO_EXECUTE = '/usr/bin/yadtshell'
DEFAULT_TARGETS           = set()
DEFAULT_TARGETS_DIRECTORY = '/etc/yadtshell/targets/'

SECTION_BROADCASTER = 'broadcaster'
SECTION_RECEIVER    = 'receiver'
SECTION_GRAPHITE    = 'graphite'


class ConfigurationException (Exception):
    """
        to be raised when an configuration error occurs.
    """


class YadtConfigParser (object):
    def __init__(self):
        """
            Creates instance of SafeConfigParser which will be used to parse
            the configuration file.
        """
        self._parser = SafeConfigParser()


    def get_option(self, section, option, default_value):
        """
            @return: the option from the section if it exists,
                     otherwise the given default value.
        """
        if self._parser.has_option(section, option):
            return self._parser.get(section, option)

        return default_value


    def get_option_as_yes_or_no_boolean(self, section, option, default_value):
        """
            @return: the boolean option from the section if it exists,
                     otherwise the given default value.
        """
        option_value = self.get_option(section, option, default_value)

        if option_value != 'yes' and option_value != 'no':
            raise ConfigurationException('Option %s in section %s expected "yes" or "no", but got %s'
                                         % (option, section, option_value))

        return option_value == 'yes'


    def get_option_as_int(self, section, option, default_value):
        """
            @return: the integer option from the section if it exists,
                     otherwise the given default value.
        """
        option_value = self.get_option(section, option, default_value)

        if not option_value.isdigit():
            raise ConfigurationException('Option %s in section %s expected a integer value, but got %s'
                                         % (option, section, option_value))
        return int(option_value)


    def get_option_as_list(self, section, option, default_value):
        """
            @return: the option (list of strings) from the section if it exists
                     otherwise the given default value.
        """
        option_value = self.get_option(section, option, '')

        if option_value == '':
            return default_value

        list_of_unstripped_options = option_value.split(',')

        result = []
        for unstripped_option in list_of_unstripped_options:
            result.append(unstripped_option.strip())

        return result


    def get_option_as_set(self, section, option, default_value):
        """
            @return: the option (set of strings) from the section if it exists,
                     otherwise the given default value.
        """
        option_values = self.get_option_as_list(section, option, default_value)
        return set(option_values)


    def read_configuration_file(self, filename):
        """
            reads the file into the parser. Will exit with error code 1 if
            the configuration file does not exist.
        """
        if not os.path.exists(filename):
            sys.stderr.write('Configuration file "%s" does not exist.\n' % filename)
            exit(1)
            return

        sys.stdout.write('Loading configuration file "%s"' % filename)
        self._parser.read([filename])


class ReceiverConfigLoader (object):
    """
        uses a SafeConfigParser to offer some convenience methods.
    """

    def __init__(self):
        """
            Creates instance of YadtConfigParser which will be used to parse
            the configuration file.
        """
        self._parser = YadtConfigParser()


    def get_broadcaster_host(self):
        """
            @return: the broadcaster host from the configuration file,
                     otherwise DEFAULT_BROADCASTER_HOST.
        """
        return self._parser.get_option(SECTION_BROADCASTER, 'host', DEFAULT_BROADCASTER_HOST)


    def get_broadcaster_port(self):
        """
            @return: the broadcaster port from the configuration file as int,
                     otherwise DEFAULT_BROADCASTER_PORT.
        """
        return self._parser.get_option_as_int(SECTION_BROADCASTER, 'port', DEFAULT_BROADCASTER_PORT)


    def get_graphite_active(self):
        """
            @return: the graphite active porperty from the configuration file
                     as bool, otherwise DEFAULT_GRAPHITE_ACTIVE.
        """
        return self._parser.get_option_as_yes_or_no_boolean(SECTION_GRAPHITE, 'active', DEFAULT_GRAPHITE_ACTIVE)


    def get_graphite_host(self):
        """
            @return: the graphite host from the configuration file,
                     otherwise DEFAULT_GRAPHITE_HOST.
        """
        return self._parser.get_option(SECTION_GRAPHITE, 'host', DEFAULT_GRAPHITE_HOST)


    def get_graphite_port(self):
        """
            @return: the graphite port from the configuration file as int,
                     otherwise DEFAULT_GRAPHITE_PORT.
        """
        return self._parser.get_option_as_int(SECTION_GRAPHITE, 'port', DEFAULT_GRAPHITE_PORT)


    def get_hostname(self):
        """
            @return: if a hostname is given in the configration file it will
                     return it, otherwise it will return the host name as
                     given by socket.gethostname
        """
        return self._parser.get_option(SECTION_RECEIVER, 'hostname', socket.gethostname())


    def get_log_filename(self):
        """
            @return: the log filename from the configuration file if given,
                     otherwise DEFAULT_LOG_FILENAME
        """
        return self._parser.get_option(SECTION_RECEIVER, 'log_filename', DEFAULT_LOG_FILENAME)


    def get_python_command(self):
        """
            @return: the python command from the configuration file if given,
                     otherwise DEFAULT_PYTHON_COMMAND
        """
        return self._parser.get_option(SECTION_RECEIVER, 'python_command', DEFAULT_PYTHON_COMMAND)


    def get_script_to_execute(self):
        """
            @return: if the configuration file provides a script to execute,
                     otherwise DEFAULT_SCRIPT_TO_EXECUTE.
        """
        return self._parser.get_option(SECTION_RECEIVER, 'script_to_execute', DEFAULT_SCRIPT_TO_EXECUTE)


    def get_targets(self):
        """
            @return: the set of targets given in the configuration file,
                     otherwise DEFAULT_TARGETS.
        """
        return self._parser.get_option_as_set(SECTION_RECEIVER, 'targets', DEFAULT_TARGETS)


    def get_targets_directory(self):
        """
            @return: the targets directory from the configuration file,
                     otherwise DEFAULT_TARGETS_DIRECTORY.
        """
        return self._parser.get_option(SECTION_RECEIVER, 'targets_directory', DEFAULT_TARGETS_DIRECTORY)

    def read_configuration_file (self, filename):
        """
            Reads the given configuration file. Uses the YadtConfigParser.
            
            @return: configuration dictionary
        """
        return self._parser.read_configuration_file(filename)


def load(filename):
    """
        loads configuration from a file.

        @return: Configuration object containing the data from the file.
    """
    parser = ReceiverConfigLoader()
    parser.read_configuration_file(filename)

    configuration = {'broadcaster_host' : parser.get_broadcaster_host(),
                     'broadcaster_port' : parser.get_broadcaster_port(),
                     'graphite_active'  : parser.get_graphite_active(),
                     'graphite_host'    : parser.get_graphite_host(),
                     'graphite_port'    : parser.get_graphite_port(),
                     'hostname'         : parser.get_hostname(),
                     'log_filename'     : parser.get_log_filename(),
                     'python_command'   : parser.get_python_command(),
                     'script_to_execute': parser.get_script_to_execute(),
                     'targets'          : parser.get_targets(),
                     'targets_directory': parser.get_targets_directory()}

    return configuration



#!/usr/bin/env python

from setuptools import setup

if __name__ == '__main__':
    setup(
          name = 'yadtreceiver',
          version = '0.1.10',
          description = 'Executes yadtshell commands triggered by a yadtbroadcaster.',
          long_description = '''''',
          author = "Arne Hilmann, Maximilien Riehl, Michael Gruber",
          author_email = "arne.hilmann@gmail.com, maximilien.riehl@gmail.com, aelgru@gmail.com",
          license = 'GNU GPL v3',
          url = 'https://github.com/yadt/yadtreceiver',
          scripts = [],
          packages = ['yadtreceiver'],
          classifiers = ['Development Status :: 3 - Alpha', 'Programming Language :: Python'],
          data_files = [('/etc/twisted-taps/', ['yadtreceiver/yadtreceiver.tac']), ('/etc/init.d/', ['yadtreceiver/yadtreceiver'])],
          
          install_requires = [ "PyYAML", "Twisted", "yadtbroadcast-client" ],
          
          zip_safe=True
    )



#   yadtreceiver
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    A broadcaster event.
"""

__author__ = 'Arne Hilmann, Michael Gruber, Maximilien Riehl'

FAILED = 'failed'
FINISHED = 'finished'
STARTED = 'started'

ATTRIBUTE_ARGUMENTS = 'args'
ATTRIBUTE_COMMAND = 'cmd'
ATTRIBUTE_MESSAGE = 'message'
ATTRIBUTE_STATE = 'state'
ATTRIBUTE_TYPE = 'id'
ATTRIBUTE_PAYLOAD = 'payload'

PAYLOAD_ATTRIBUTE_URI = 'uri'
PAYLOAD_ATTRIBUTE_STATE = 'state'

TYPE_COMMAND = 'cmd'
TYPE_FULL_UPDATE = 'full-update'
TYPE_REQUEST = 'request'
TYPE_SERVICE_CHANGE = 'service-change'
KNOWN_EVENT_TYPES = [TYPE_COMMAND, TYPE_FULL_UPDATE, TYPE_REQUEST, TYPE_SERVICE_CHANGE]


class IncompleteEventDataException(Exception):
    """
        to be raised when an error during validation of an event occurs.
    """

    def __init__(self, event, attribute_name):
        error_message = 'Event "{1}" on target {0} is missing attribute "{2}", event dump : {3}'.format(event.target,
                                                                                            event.event_type,
                                                                                            attribute_name,
                                                                                            event.data)
        super(IncompleteEventDataException, self).__init__(error_message)

class InvalidEventTypeException(Exception):
    """
        to be raised when an invalid event is instantiated
    """
    def __init__(self, event, event_type):
        error_message = 'Event "{1}" on target {0} has invalid type, event dump : {2}'.format(event.target,
                                                                             event_type,
                                                                             event.data)
        super(InvalidEventTypeException, self).__init__(error_message)

class PayloadIntegrityException(Exception):
    """
        to be raised when a payload cannot be validated
    """

    def __init__(self, event, payload_attribute_name):
        error_message = 'Event "{1}" on target {0} is missing attribute {2} in payload, event dump : {3}'.format(
                                                                                               event.target,
                                                                                               event.event_type,
                                                                                               payload_attribute_name,
                                                                                               event.data)
        super(PayloadIntegrityException, self).__init__(error_message)

class Event (object):

    def __init__(self, target, data):
        self.target = target
        self.data = data
        self.event_type = self._ensure_is_valid_event_type()

        if self.is_a_request:
            self._initialize_request()

        if self.is_a_service_change:
            self._initialize_service_change()

        if self.is_a_command:
            self._initialize_command()

    def _initialize_request(self):
        self.command = self._ensure_attribute_in_data(ATTRIBUTE_COMMAND)
        self.arguments = self._ensure_attribute_in_data(ATTRIBUTE_ARGUMENTS)

    def _initialize_command(self):
        self.command = self._ensure_attribute_in_data(ATTRIBUTE_COMMAND)
        self.state = self._ensure_attribute_in_data(ATTRIBUTE_STATE)

        if ATTRIBUTE_MESSAGE in self.data:
            self.message = self.data[ATTRIBUTE_MESSAGE]

    def _initialize_service_change(self):
        payload = self._ensure_attribute_in_data(ATTRIBUTE_PAYLOAD)

        self.service_states = self._extract_service_states_from_payload(payload)

    def _extract_service_states_from_payload(self, payload):
        service_states = []

        for payload_entry in payload:
            uri = self._ensure_payload_entry_contains_attribute(payload_entry, PAYLOAD_ATTRIBUTE_URI)
            state = self._ensure_payload_entry_contains_attribute(payload_entry, PAYLOAD_ATTRIBUTE_STATE)
            service_states.append(self.ServiceState(uri, state))
        return service_states

    def _ensure_payload_entry_contains_attribute(self, payload_entry, attribute_name):
        if not attribute_name in payload_entry:
            raise PayloadIntegrityException(self, attribute_name)
        return payload_entry[attribute_name]

    def _ensure_is_valid_event_type(self):
        if ATTRIBUTE_TYPE not in self.data:
            raise InvalidEventTypeException(self, None)

        event_type = self.data[ATTRIBUTE_TYPE]
        if not event_type in KNOWN_EVENT_TYPES:
            raise InvalidEventTypeException(self, event_type)
        return event_type

    def _ensure_attribute_in_data(self, attribute_name):
        if not attribute_name in self.data:
            raise IncompleteEventDataException(self, attribute_name)
        return self.data[attribute_name]

    @property
    def is_a_request(self):
        return self.event_type == TYPE_REQUEST

    @property
    def is_a_full_update(self):
        return self.event_type == TYPE_FULL_UPDATE

    @property
    def is_a_service_change(self):
        return self.event_type == TYPE_SERVICE_CHANGE

    @property
    def is_a_command(self):
        return self.event_type == TYPE_COMMAND

    def __str__(self):
        if self.is_a_request:
            return 'target[{0}] requested command "{1}" using arguments "{2}"'.format(self.target, self.command, self.arguments)

        if self.is_a_full_update:
            return 'target[{0}] full update of status information.'.format(self.target)

        if self.is_a_service_change:
            state_changes = ', '.join(map(str, self.service_states))
            return 'target[{0}] services changed: {1}'.format(self.target, state_changes)

        if self.is_a_command:
            if hasattr(self, 'message') and self.message is not None:
                return '(broadcaster) target[{0}] command "{1}" {2}: {3}'.format(self.target, self.command, self.state, self.message)
            else:
                return '(broadcaster) target[{0}] command "{1}" {2}.'.format(self.target, self.command, self.state)

        raise NotImplementedError('Unknown event type {0}'.format(self.event_type))

    class ServiceState (object):
        def __init__(self, uri, state):
            self.uri = uri
            self.state = state

        def __str__(self):
            return '{0} is {1}'.format(self.uri, self.state)


#   yadtreceiver
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    Provides a function to send graphite a update notification.
    
    For more information about graphite please visit:
    http://graphite.wikidot.com/
"""

__author__ = 'Michael Gruber'

from socket import socket
from time import time
from twisted.python import log


def send_update_notification_to_graphite(target, host, port):
    """
        Sends a update notification to the graphite server at host:port.
    """
    try:
        graphite_socket  = socket()
        graphite_address = (host, port)
        graphite_socket.connect(graphite_address)

        timestamp = int(time())
        log.msg('sending update of target %s notification to graphite on %s:%s' % (target, host, port))
        graphite_socket.send('yadt.%s.update 1 %d\n' % (target, timestamp))

    except Exception as e:
        log.err(e)

    finally:
        if graphite_socket:
            graphite_socket.close()
